@@ -11,6 +11,7 @@ import enum
 import itertools
 import logging
 import os
+import pprint
 import typing
 
 import ccc.oci
@@ -19,14 +20,18 @@ import cnudie.replicate
 import cnudie.retrieve
 import container.util
 import gci.componentmodel as cm
+import oci
 import product.v2
 import yaml
 
 import ctt.filters as filters
 import ctt.processing_model as processing_model
 import ctt.processors as processors
-import ctt.uploaders as uploaders
 from ctt.rbsc_bom import BOMEntry, BOMEntryType, buildAndApplyBOM
+import ctt.uploaders as uploaders
+import ctt.util as ctt_util
+
+original_tag_label_name = 'cloud.gardener.cnudie/migration/original_tag'
 
 logger = logging.getLogger(__name__)
 
@@ -271,7 +276,8 @@ def create_jobs(
 # uploads a single OCI artifact and returns the content digest
 def process_upload_request(
     upload_request: processing_model.ContainerImageUploadRequest,
-    upload_mode_images=product.v2.UploadMode.SKIP
+    upload_mode_images=product.v2.UploadMode.SKIP,
+    replication_mode=oci.ReplicationMode.PREFER_MULTIARCH,
 ) -> str:
     tgt_ref = upload_request.target_ref
 
@@ -289,6 +295,7 @@ def process_upload_request(
         source_ref=src_ref,
         target_ref=tgt_ref,
         remove_files=upload_request.remove_files,
+        mode=replication_mode,
     )
 
     logger.info(f'finished processing {src_ref} -> {tgt_ref=}')
@@ -307,14 +314,36 @@ def replace_tag_with_digest(image_reference: str, docker_content_digest: str) ->
     return f'{src_name}@{docker_content_digest}'
 
 
+def labels_with_original_tag(
+    resource: cm.Resource,
+    src_ref: str,
+) -> typing.Sequence[cm.Label]:
+    last_part = src_ref.split('/')[-1]
+    if '@' in last_part:
+        raise RuntimeError(f'cannot extract tag from resource that is referenced via digest. {resource=}')
+
+    _, src_tag = src_ref.rsplit(':', 1)
+    original_tag_label = cm.Label(
+        name=original_tag_label_name,
+        value=src_tag,
+    )
+    src_labels = resource.labels or []
+    return ctt_util.add_label(
+        src_labels=src_labels,
+        label=original_tag_label,
+    )
+
+
 def access_resource_via_digest(res: cm.Resource, docker_content_digest: str) -> cm.Resource:
     if res.access.type is cm.AccessType.OCI_REGISTRY:
+        updated_labels = labels_with_original_tag(res, res.access.imageReference)
         digest_ref = replace_tag_with_digest(res.access.imageReference, docker_content_digest)
         digest_access = cm.OciAccess(
             cm.AccessType.OCI_REGISTRY,
             imageReference=digest_ref,
         )
     elif res.access.type is cm.AccessType.RELATIVE_OCI_REFERENCE:
+        updated_labels = labels_with_original_tag(res, res.access.reference)
         digest_ref = replace_tag_with_digest(res.access.reference, docker_content_digest)
         digest_access = cm.RelativeOciAccess(
             cm.AccessType.RELATIVE_OCI_REFERENCE,
@@ -326,6 +355,7 @@ def access_resource_via_digest(res: cm.Resource, docker_content_digest: str) ->
     return dataclasses.replace(
         res,
         access=digest_access,
+        labels=updated_labels,
     )
 
 
@@ -337,15 +367,14 @@ def process_images(
     upload_mode=None,
     upload_mode_cd=product.v2.UploadMode.SKIP,
     upload_mode_images=product.v2.UploadMode.SKIP,
+    replication_mode=oci.ReplicationMode.PREFER_MULTIARCH,
     replace_resource_tags_with_digests=False,
     skip_cd_validation=False,
 ):
+    logger.info(pprint.pformat(locals()))
+
     if processing_mode is ProcessingMode.DRY_RUN:
         ci.util.warning('dry-run: not downloading or uploading any images')
-    else:
-        logger.info(f'using upload_mode_cd {upload_mode_cd}')
-        logger.info(f'using upload_mode_images {upload_mode_images}')
-        logger.info(f'using skip_cd_validation {skip_cd_validation}')
 
     if upload_mode_images is product.v2.UploadMode.FAIL:
         raise NotImplementedError('upload-mode-image=fail is not a valid argument.')
@@ -372,7 +401,11 @@ def process_images(
     def process_job(processing_job: processing_model.ProcessingJob):
         # do actual processing
         if processing_mode is ProcessingMode.REGULAR:
-            docker_content_digest = process_upload_request(processing_job.upload_request, upload_mode_images)
+            docker_content_digest = process_upload_request(
+                upload_request=processing_job.upload_request,
+                upload_mode_images=upload_mode_images,
+                replication_mode=replication_mode,
+            )
 
             if replace_resource_tags_with_digests:
                 if not docker_content_digest:
@@ -575,6 +608,13 @@ def main():
     parser.add_argument('-r', '--replace-resource-tags-with-digests', action='store_true',
                         help='replace tags with digests for resources that are accessed via OCI references'
                         )
+    parser.add_argument('--replication-mode',
+                        help='replication mode for OCI resources',
+                        choices=[
+                            mode.value for _, mode in oci.ReplicationMode.__members__.items()
+                        ],
+                        default=oci.ReplicationMode.PREFER_MULTIARCH
+                        )
 
     parsed = parser.parse_args()
 
@@ -616,6 +656,7 @@ def main():
         processing_mode=processing_mode,
         upload_mode_cd=product.v2.UploadMode(parsed.upload_mode_cd),
         upload_mode_images=product.v2.UploadMode(parsed.upload_mode_images),
+        replication_mode=oci.ReplicationMode(parsed.replication_mode),
         replace_resource_tags_with_digests=parsed.replace_resource_tags_with_digests,
         skip_cd_validation=parsed.skip_cd_validation,
     )
